/*
* This confidential and proprietary software may be used only as
* authorized by licensing agreement from Multicamera.Systems LLC
* Copyright (C) 2017-2020 Multicamera.Systems LLC - All Rights Reserved
* You should have received a copy of the our license with this file.
* If not, please write to: info@multicamera.systems
*
* This code and information is provided "AS IS" without warranty of any kind,
* either expressed or implied, including but not limited to the implied
* warranties of merchantability and/or fitness for a particular purpose.
*/

#pragma once
#include "TSList.h"


#define FRAME_DESC_QUEUE_ID__MAX    8

#define INVALID_REF     ((uintptr_t)-1LL)



// "Frame" is atomic piece of data, i.e.:
// - single image frame for Camera device;
// - XYZ tripple for lidar
// - etc

// the following structure is the descriptor of the data representing one or more frames


class FRAME_DESC : public ObjMLink<FRAME_DESC, FRAME_DESC_QUEUE_ID__MAX>
{
public:
    uint64_t ullFrameNumber;

    void *pData;                            // pointer to the frame data

    uint32_t uiFrameSize;                   // size of the single frame

    uint32_t ullReserved1;

    uint32_t uiFrameCount;                  // number of frames in the data pointer by pData

    uint64_t ullTimeStamp;                  // time stamp of the (first) frame 

    uint64_t ullReserved2; 

    uintptr_t DataRef[2];                   // additional reference of a buffer pointed by pData

    //

    std::atomic<uint32_t> uiRefCountFlags;  // reference counter or reference flags

public:
    FRAME_DESC() : ObjMLink<FRAME_DESC, FRAME_DESC_QUEUE_ID__MAX>(this),
                   ullFrameNumber(0), pData(NULL), uiFrameSize(0), uiFrameCount(1), ullTimeStamp(0)
    {
        uiRefCountFlags.store(0);
        memset(&DataRef, -1, sizeof(DataRef));
    }

    FRAME_DESC(const FRAME_DESC& src) : ObjMLink<FRAME_DESC, FRAME_DESC_QUEUE_ID__MAX>(this),
                                        ullFrameNumber(src.ullFrameNumber), pData(src.pData),
                                        uiFrameSize(src.uiFrameSize), uiFrameCount(src.uiFrameCount), ullTimeStamp(src.ullTimeStamp)
    {
        uiRefCountFlags.store(src.uiRefCountFlags);
        memcpy(&DataRef, src.DataRef, sizeof(DataRef));
    }
};




class FRAME_DESC_QUEUE : public TSList<FRAME_DESC>
{
public:
    FRAME_DESC_QUEUE() : TSList<FRAME_DESC>() {}
    FRAME_DESC_QUEUE(const char *szName, void *pOwner) : TSList<FRAME_DESC>(szName, pOwner) {}
};
