/*
* This confidential and proprietary software may be used only as
* authorized by licensing agreement from Multicamera.Systems LLC
* Copyright (C) 2017-2020 Multicamera.Systems LLC - All Rights Reserved
* You should have received a copy of the our license with this file.
* If not, please write to: info@multicamera.systems
*
* This code and information is provided "AS IS" without warranty of any kind,
* either expressed or implied, including but not limited to the implied
* warranties of merchantability and/or fitness for a particular purpose.
*/

#pragma once
#include <stdint.h>


typedef enum
{
    PIXEL_FORMAT__UNKNOWN,

    PIXEL_FORMAT__MONO,

    PIXEL_FORMAT__BAYER_RG,
    PIXEL_FORMAT__BAYER_GR,
    PIXEL_FORMAT__BAYER_BG,
    PIXEL_FORMAT__BAYER_GB,

    PIXEL_FORMAT__RGBX = 8,
    PIXEL_FORMAT__BGRX,

    PIXEL_FORMAT__YUV_PLANAR_420 = 16,
    PIXEL_FORMAT__YUV_PLANAR_444 = 21,

    PIXEL_FORMAT__YUV_SEMI_PLANAR_420,      // NV12

} PIXEL_FORMAT;




typedef enum : uint8_t
{
    COMPRESSION_TYPE__RESERVED,
    COMPRESSION_TYPE__UNCOMPRESSED,
    COMPRESSION_TYPE__LZW,  // not supported in public release
    COMPRESSION_TYPE__JPEG, // not supported in public release
    COMPRESSION_TYPE__H264,
    COMPRESSION_TYPE__H265,
    COMPRESSION_TYPE__VP8,
    COMPRESSION_TYPE__VP9,
    COMPRESSION_TYPE__AV1,  // not supported in public release
    COMPRESSION_TYPE__VC1,
    COMPRESSION_TYPE__MPEG2,
    COMPRESSION_TYPE__MAX

} COMPRESSION_TYPE;



typedef struct
{
    COMPRESSION_TYPE Type;

    uint8_t ucGOPSize;

    uint8_t ucChromasPerLumaQuad;    // 0 - mono, 1 : 420, 2: 422, 4 : 444

    uint8_t ucComponentSize;

} COMPRESSION_INFO;



typedef enum
{
    COMPRESSION_QUALITY_ID__LOW,
    COMPRESSION_QUALITY_ID__MEDIUM,
    COMPRESSION_QUALITY_ID__HIGH,
    COMPRESSION_QUALITY_ID__MAX

} COMPRESSION_QUALITY_ID;


#define COMPRESSION_FLAG__QUALITY_F_VALID   0x00000001
#define COMPRESSION_FLAG__QUALITY_ID_VALID  0x00000002
#define COMPRESSION_FLAG__BITRATE_VALID     0x00000004
#define COMPRESSION_FLAG__BITRATE_VBR       0x00000008


typedef struct
{
    COMPRESSION_INFO Info;

    uint32_t uiFlags;       // validity and other flags

    COMPRESSION_QUALITY_ID QualityID;

    float fQuality;         // jpeg
    uint32_t uiBitRate;     // h.264, h.265

    uint32_t uiReserved;

} COMPRESSION_SETTINGS;



typedef struct
{
    PIXEL_FORMAT PixelFormat;

    uint32_t uiComponentSize;
    uint32_t uiPixelSize;

} IMAGE_FORMAT;



typedef struct
{
    IMAGE_FORMAT Format;

    COMPRESSION_INFO Compression;

    uint32_t uiWidth;
    uint32_t uiHeight;

    float fFrameRate;
    
    uint32_t uiNotUsed[10];

} INPUT_STREAM_PARAMETERS;



typedef struct
{
    uint32_t uiTemperature = 0;
    struct
    {
        float fRGain = 1.f;
        float fBGain = 1.f;
    } TintCorrection;

} WHITE_BALANCE;



typedef struct
{
    struct
    {
        bool bFixupNALUSize;    // if true 00-00-00-01 NALU header bytes will be substitured with the actual NALU size
        bool bGenerateSPSPPS;   // if true, SPS/PPS will be generated in the bitstream (first GOP)

        uint16_t vps_size;
        uint8_t vps[256];

        uint16_t sps_size;
        uint8_t sps[1024];

        uint16_t pps_size;
        uint8_t pps[256];

    } h26x;

} COMPRESSION_CODEC_INFO;



typedef struct
{
    uint32_t uiWidth;       // output dimensions
    uint32_t uiHeight;

    struct
    {
        uint32_t uiX;
        uint32_t uiY;
        uint32_t uiWidth;
        uint32_t uiHeight;
    } Window;               // window into input image

    COMPRESSION_SETTINGS Compression;

    COMPRESSION_CODEC_INFO CodecInfo;   // Should be filled on input if input is compressed. Will be filled on OUTPUT with VPS/SPS/PPS info

    uint32_t uiNotUsed;

    WHITE_BALANCE WhiteBalance; // used for Bayer input only
    
    uint32_t uiPipelineDepth;   // Filled on OUTPUT: this value shows how many frames it may be buffered before starting to produce output
                                // a client can use this field to calculate number of its input buffers needed for continous operation

} OUTPUT_STREAM_PARAMETERS;

