/*
* This confidential and proprietary software may be used only as
* authorized by licensing agreement from Multicamera.Systems LLC
* Copyright (C) 2017-2020 Multicamera.Systems LLC - All Rights Reserved
* You should have received a copy of the our license with this file.
* If not, please write to: info@multicamera.systems
*
* This code and information is provided "AS IS" without warranty of any kind,
* either expressed or implied, including but not limited to the implied
* warranties of merchantability and/or fitness for a particular purpose.
*/

#pragma once

#ifdef _WIN32

#include <windows.h>

#define OS_GUARDED_CODE_SECTION                     CRITICAL_SECTION
#define OS_InitializeGuardedCodeSection(pointer)    InitializeCriticalSection(pointer)
#define OS_EnterGuardedCodeSection(pointer)         EnterCriticalSection(pointer)
#define OS_TryEnterGuardedCodeSection(pointer)      TryEnterCriticalSection(pointer)
#define OS_LeaveGuardedCodeSection(pointer)         LeaveCriticalSection(pointer)
#define OS_DeleteGuardedCodeSection(pointer)        DeleteCriticalSection(pointer)


#ifdef _DEBUG
#define OS_DebugBreak()                             DebugBreak()
#else
#define OS_DebugBreak()
#endif

#else

#include <pthread.h>

typedef pthread_mutex_t OS_GUARDED_CODE_SECTION;

void OS_InitializeGuardedCodeSection(OS_GUARDED_CODE_SECTION* pointer)
{
    *pointer = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
}

void OS_EnterGuardedCodeSection(OS_GUARDED_CODE_SECTION* pointer)
{
    pthread_mutex_lock(pointer);
}

BOOL OS_TryEnterGuardedCodeSection(OS_GUARDED_CODE_SECTION* pointer)
{
    int st;
    st = pthread_mutex_trylock(pointer);
    if (st == EBUSY)
    {
        return 0;
    }

    return 1;
}

void OS_LeaveGuardedCodeSection(OS_GUARDED_CODE_SECTION* pointer)
{
    pthread_mutex_unlock(pointer);
}

void OS_DeleteGuardedCodeSection(OS_GUARDED_CODE_SECTION* pointer)
{
    return;
}

void OS_DebugBreak();

#endif
